{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit ExplorU;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ImgList, ComCtrls, DM_TLB, StdCtrls, Buttons, AppEvnts, 
  {$ifdef VER140}Variants, {$endif}Spin;

type
  TExplorerForm = class(TForm)
    TreeView: TTreeView;
    ImageList: TImageList;
    Notebook: TNotebook;
    Label1: TLabel;
    DocumentCountLabel: TLabel;
    Label2: TLabel;
    MessageHintEdit: TEdit;
    MessageButton: TButton;
    HintButton: TButton;
    Label3: TLabel;
    ActionComboBox: TComboBox;
    VisibleCheckBox: TCheckBox;
    EnabledCheckBox: TCheckBox;
    CheckedCheckBox: TCheckBox;
    NotesMemo: TMemo;
    LoadNotesSpeedButton: TSpeedButton;
    OpenDialog: TOpenDialog;
    SaveNotesSpeedButton: TSpeedButton;
    ClearNotesSpeedButton: TSpeedButton;
    SaveDialog: TSaveDialog;
    LineNotesSpinEdit: TSpinEdit;
    Label4: TLabel;
    NotesEdit: TEdit;
    AddNotesButton: TButton;
    InsertNotesButton: TButton;
    DeleteNotesButton: TButton;
    ExecCommandBitBtn: TBitBtn;
    ApplicationMemo: TMemo;
    RecordPointButton: TButton;
    SetExpSettingsButton: TButton;
    CreateDocumentSpeedButton: TSpeedButton;
    WindowCaptionLabel: TLabel;
    WindowStateRadioGroup: TRadioGroup;
    WindowRecordingCheckBox: TCheckBox;
    ActivateButton: TButton;
    CloseButton: TButton;
    SaveUndoButton: TButton;
    Label5: TLabel;
    ColSpinEdit: TSpinEdit;
    XColCheckBox: TCheckBox;
    YColCheckBox: TCheckBox;
    Label6: TLabel;
    ColLabelEdit: TEdit;
    Label7: TLabel;
    LineSpinEdit: TSpinEdit;
    ScrollButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure LoadNotesSpeedButtonClick(Sender: TObject);
    procedure NotesMemoChange(Sender: TObject);
    procedure ActionComboBoxChange(Sender: TObject);
    procedure VisibleCheckBoxClick(Sender: TObject);
    procedure EnabledCheckBoxClick(Sender: TObject);
    procedure CheckedCheckBoxClick(Sender: TObject);
    procedure SaveNotesSpeedButtonClick(Sender: TObject);
    procedure ClearNotesSpeedButtonClick(Sender: TObject);
    procedure AddNotesButtonClick(Sender: TObject);
    procedure InsertNotesButtonClick(Sender: TObject);
    procedure DeleteNotesButtonClick(Sender: TObject);
    procedure LineNotesSpinEditChange(Sender: TObject);
    procedure NotesEditChange(Sender: TObject);
    procedure MessageButtonClick(Sender: TObject);
    procedure HintButtonClick(Sender: TObject);
    procedure ExecCommandBitBtnClick(Sender: TObject);
    procedure RecordPointButtonClick(Sender: TObject);
    procedure SetExpSettingsButtonClick(Sender: TObject);
    procedure CreateDocumentSpeedButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure SaveUndoButtonClick(Sender: TObject);
    procedure WindowStateRadioGroupClick(Sender: TObject);
    procedure WindowRecordingCheckBoxClick(Sender: TObject);
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure ActivateButtonClick(Sender: TObject);
    procedure ColSpinEditChange(Sender: TObject);
    procedure ColLabelEditChange(Sender: TObject);
    procedure XColCheckBoxClick(Sender: TObject);
    procedure YColCheckBoxClick(Sender: TObject);
    procedure ScrollButtonClick(Sender: TObject);
  private
    { Private declarations }
    DM: TDMApplication;
    Root: TTreeNode;
    procedure CreateDocument(Sender: TObject; var Document: OleVariant);
    procedure CloseDocument(Sender: TObject; var Document: OleVariant);
    procedure Command(Sender: TObject; var Command: OleVariant);
    function GetDocument: IDMDocument;
    function GetWorksheet: IDMWorksheet;
  public
    { Public declarations }
  end;

var
  ExplorerForm: TExplorerForm;

implementation

{$R *.DFM}

////////// Service

function TExplorerForm.GetDocument: IDMDocument;
begin                                 // selected node MUST POINT TO DOCUMENT!
  Result:=IDispatch(TreeView.Selected.Data) as IDMDocument;
end;

function TExplorerForm.GetWorksheet: IDMWorksheet;
begin                                // selected node MUST POINT TO WORKSHEET!
  Result:=(IDispatch(TreeView.Selected.Parent.Data) as IDMDocument).Worksheet
  as IDMWorksheet;
end;

////////// SERVER EVENT HANDLERS //////////////////

procedure TExplorerForm.CreateDocument(Sender: TObject;
  var Document: OleVariant);
var Doc,Plot: TTreeNode; I: integer;
begin
  Doc:=TreeView.Items.AddChild(Root,'DMDocument');
  with Doc do
  begin
    ImageIndex:=2; SelectedIndex:=2;
    Data:=pointer(TVarData(Document).VDispatch);
    IDispatch(Data)._AddRef;
  end;
  with TreeView.Items.AddChild(Doc,'DMWorksheet') do
  begin ImageIndex:=4; SelectedIndex:=4; end;
  with TreeView.Items.AddChild(Doc,'DMContainer') do
  begin ImageIndex:=3; SelectedIndex:=3; end;
  Plot:=TreeView.Items.AddChild(Doc,'DMPlot');
  with Plot do begin ImageIndex:=5; SelectedIndex:=5; end;
  with TreeView.Items.AddChild(Plot,'DMAxis') do
  begin ImageIndex:=7; SelectedIndex:=7; end;
  with TreeView.Items.AddChild(Plot,'DMAxis') do
  begin ImageIndex:=8; SelectedIndex:=8; end;
  for I:=0 to Document.Plot.SerieCount-1 do
  with TreeView.Items.AddChild(Plot,'DMSerie-'+IntToStr(I)) do
  begin ImageIndex:=6; SelectedIndex:=6; end;
  Doc.Expand(true); Doc.Selected:=true;
end;

procedure TExplorerForm.CloseDocument(Sender: TObject;
  var Document: OleVariant);
var I: integer;
begin
  for I:=0 to Root.Count-1 do with Root.Item[I] do
  if (ImageIndex=2) and (Data=pointer(TVarData(Document).VDispatch)) then
  begin IDispatch(Data)._Release; Delete; Break; end;
end;

procedure TExplorerForm.Command(Sender: TObject; var Command: OleVariant);
begin MessageDlg(Command,mtInformation,[mbOk],0); end;

////////// GENERAL HANDLERS /////////////////////////

procedure TExplorerForm.FormCreate(Sender: TObject);
begin
  DM:=TDMApplication.Create(Self);
  DM.OnCreateDocument:=CreateDocument;
  DM.OnCloseDocument:=CloseDocument;
  DM.OnCommand:=Command;
  DM.Connect;
  TreeView.Items.Clear;
  Root:=TreeView.Items.Add(nil,'DMApplication');
  Root.ImageIndex:=0; Root.SelectedIndex:=0;
  with TreeView.Items.AddChild(Root,'DMNotes') do
  begin ImageIndex:=1; SelectedIndex:=1; end;
  Root.Expand(true);
end;

procedure TExplorerForm.TreeViewChange(Sender: TObject; Node: TTreeNode);
begin
  case Node.ImageIndex of
  0:
  begin
    Notebook.PageIndex:=0;
    ActionComboBox.ItemIndex:=0; ActionComboBoxChange(Sender);
    DocumentCountLabel.Caption:=IntToStr(DM.DocumentCount);
  end;
  1:
  begin
    Notebook.PageIndex:=1;
    LineNotesSpinEdit.MaxValue:=(DM.Notes as IDMNotes).LineCount-1;
    NotesMemo.Lines.Text:=(DM.Notes as IDMNotes).Text;
  end;
  2:
  with GetDocument do begin
    Notebook.PageIndex:=2;
    WindowCaptionLabel.Caption:=WindowCaption;
    WindowStateRadioGroup.ItemIndex:=WindowState;
    WindowRecordingCheckBox.Checked:=IsRecording;
  end;
  4:
  {with GetWorksheet do }begin
    Notebook.PageIndex:=3; ColSpinEditChange(Sender);
  end;
  end;{case}
end;

////////////// MEMBER CONTROLS HANDLERS ///////////////////////////

///// DMApplication

procedure TExplorerForm.ActionComboBoxChange(Sender: TObject);
begin
  VisibleCheckBox.Checked:=DM.IsCommandVisible[ActionComboBox.Text];
  EnabledCheckBox.Checked:=DM.IsCommandEnabled[ActionComboBox.Text];
  CheckedCheckBox.Checked:=DM.IsCommandChecked[ActionComboBox.Text];
end;

procedure TExplorerForm.VisibleCheckBoxClick(Sender: TObject);
begin
  DM.IsCommandVisible[ActionComboBox.Text]:=VisibleCheckBox.Checked;
end;

procedure TExplorerForm.EnabledCheckBoxClick(Sender: TObject);
begin
  DM.IsCommandEnabled[ActionComboBox.Text]:=EnabledCheckBox.Checked;
end;

procedure TExplorerForm.CheckedCheckBoxClick(Sender: TObject);
begin
  DM.IsCommandChecked[ActionComboBox.Text]:=CheckedCheckBox.Checked;
end;

procedure TExplorerForm.MessageButtonClick(Sender: TObject);
begin
  DM.DisplayMessage(MessageHintEdit.Text);
end;

procedure TExplorerForm.HintButtonClick(Sender: TObject);
begin
  DM.DisplayHint(MessageHintEdit.Text);
end;

procedure TExplorerForm.ExecCommandBitBtnClick(Sender: TObject);
begin
  DM.ExecuteCommand(ActionComboBox.Text);
end;

procedure TExplorerForm.RecordPointButtonClick(Sender: TObject);
begin
  DM.Recordpoint(ApplicationMemo.Lines.Text, '1.234567 2.3456789');
end;

procedure TExplorerForm.SetExpSettingsButtonClick(Sender: TObject);
begin
  DM.SetExpSettings(ApplicationMemo.Lines.Text);
end;

procedure TExplorerForm.CreateDocumentSpeedButtonClick(Sender: TObject);
begin
  OpenDialog.Filename:='';
  if OpenDialog.Execute then DM.CreateDocument(OpenDialog.Filename)
  else DM.CreateDocument('');
end;

//////// DMNotes

procedure TExplorerForm.LoadNotesSpeedButtonClick(Sender: TObject);
begin
  OpenDialog.Filename:='';
  if OpenDialog.Execute then (DM.Notes as IDMNotes).Load(OpenDialog.Filename);
end;

procedure TExplorerForm.NotesMemoChange(Sender: TObject);
begin (DM.Notes as IDMNotes).Text:=NotesMemo.Lines.Text; end;

procedure TExplorerForm.SaveNotesSpeedButtonClick(Sender: TObject);
begin
  SaveDialog.Filename:='';
  if SaveDialog.Execute then (DM.Notes as IDMNotes).Save(SaveDialog.Filename);
end;

procedure TExplorerForm.ClearNotesSpeedButtonClick(Sender: TObject);
begin
  (DM.Notes as IDMNotes).Clear;
end;

procedure TExplorerForm.AddNotesButtonClick(Sender: TObject);
begin
  (DM.Notes as IDMNotes).AddLine(NotesEdit.Text);
end;

procedure TExplorerForm.InsertNotesButtonClick(Sender: TObject);
begin
  (DM.Notes as IDMNotes).InsertLine(LineNotesSpinEdit.Value,NotesEdit.Text);
end;

procedure TExplorerForm.DeleteNotesButtonClick(Sender: TObject);
begin
  (DM.Notes as IDMNotes).DeleteLine(LineNotesSpinEdit.Value);
end;

procedure TExplorerForm.LineNotesSpinEditChange(Sender: TObject);
begin
  NotesEdit.Text:=(DM.Notes as IDMNotes).Lines[LineNotesSpinEdit.Value];
end;

procedure TExplorerForm.NotesEditChange(Sender: TObject);
begin
  (DM.Notes as IDMNotes).Lines[LineNotesSpinEdit.Value]:=NotesEdit.Text;
end;

////// DMDocument

procedure TExplorerForm.CloseButtonClick(Sender: TObject);
begin
  GetDocument.Close;
end;

procedure TExplorerForm.SaveUndoButtonClick(Sender: TObject);
begin
  GetDocument.SaveUndoInfo;
end;

procedure TExplorerForm.ActivateButtonClick(Sender: TObject);
begin
  GetDocument.Activate;
end;

procedure TExplorerForm.WindowStateRadioGroupClick(Sender: TObject);
begin
  GetDocument.WindowState:=WindowStateRadioGroup.ItemIndex;
end;

procedure TExplorerForm.WindowRecordingCheckBoxClick(Sender: TObject);
begin
  GetDocument.IsRecording:=WindowRecordingCheckBox.Checked;
end;

/////// DMWorksheet

procedure TExplorerForm.ColSpinEditChange(Sender: TObject);
begin
  ColSpinEdit.Tag:=1;
  XColCheckBox.Checked:=GetWorksheet.XColumn=ColSpinEdit.Value;
  YColCheckBox.Checked:=GetWorksheet.YColumn=ColSpinEdit.Value;
  ColSpinEdit.Tag:=0;
  ColLabelEdit.Text:=GetWorksheet.ColumnLabels[ColSpinEdit.Value];
end;

procedure TExplorerForm.ColLabelEditChange(Sender: TObject);
begin
  GetWorksheet.ColumnLabels[ColSpinEdit.Value]:=ColLabelEdit.Text;
end;

procedure TExplorerForm.XColCheckBoxClick(Sender: TObject);
begin
  if ColSpinEdit.Tag=0 then GetWorksheet.XColumn:=ColSpinEdit.Value;
end;

procedure TExplorerForm.YColCheckBoxClick(Sender: TObject);
begin
  if ColSpinEdit.Tag=0 then GetWorksheet.YColumn:=ColSpinEdit.Value;
end;

procedure TExplorerForm.ScrollButtonClick(Sender: TObject);
begin
  GetWorksheet.Scroll(LineSpinEdit.Value);
end;

end.

