{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit ServerU;
{$B-,X+,J+}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, ScktComp, Grids, ComCtrls, ActnList;

type
  TServerForm=class(TForm)
    ConnectionsSG: TStringGrid;
    ECGroupBox: TGroupBox;
    SCGroupBox: TGroupBox;
    StartSpeedButton: TSpeedButton;
    ScanUpSpeedButton: TSpeedButton;
    ScanDownSpeedButton: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ListenSB: TSpeedButton;
    PortE: TEdit;
    UpDown: TUpDown;
    PasswordE: TEdit;
    DetachBitBtn: TBitBtn;
    ServerSocket: TServerSocket;
    ActionList: TActionList;
    ListenAction: TAction;
    DetachAction: TAction;
    CloseBitBtn: TBitBtn;
    StatusLabel: TLabel;
    RecordAction: TAction;
    Timer: TTimer;
    procedure ListenActionExecute(Sender: TObject);
    procedure DetachActionExecute(Sender: TObject);
    procedure ServerSocketClientConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerSocketClientDisconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ServerSocketClientRead(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure FormDestroy(Sender: TObject);
    procedure PasswordEChange(Sender: TObject);
    procedure DetachActionUpdate(Sender: TObject);
    procedure ListenActionUpdate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RecordActionExecute(Sender: TObject);
    procedure RecordActionUpdate(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
  public
    procedure SendData(S: string);
    procedure ShowHint(S: string);
  end;

var ServerForm: TServerForm;

implementation

{$R *.DFM}

procedure TServerForm.SendData(S: string);
var I: integer;
begin
  for I:=0 to ServerSocket.Socket.ActiveConnections-1 do {send to ALL sockets}
  ServerSocket.Socket.Connections[I].SendText(S);
end;

procedure TServerForm.ShowHint(S: string);
begin
  StatusLabel.Caption:=S; StatusLabel.Refresh;
end;

////////////// Socket events /////////////////////////////////////////////////

procedure TServerForm.ServerSocketClientConnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  ShowHint(Format('Connected to %s (%s)',
    [Socket.RemoteHost, Socket.RemoteAddress]));
  if ServerSocket.Socket.ActiveConnections>=ConnectionsSG.RowCount then
  begin                                          {limit number of connections}
    Socket.SendText(Format('message=Unable to connect more than %d sockets!',
      [ConnectionsSG.RowCount-1])); Socket.Close;
  end;
  if Socket.ReceiveText<>PasswordE.Text then               {check password...}
  begin
    Socket.SendText('message=Invalid password received!'); Socket.Close;
  end;
end;

procedure TServerForm.ServerSocketClientDisconnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  ShowHint(Format('Disconnected from %s (%s)',
  [Socket.RemoteHost, Socket.RemoteAddress]));
end;

procedure TServerForm.ServerSocketClientRead(Sender: TObject;
  Socket: TCustomWinSocket);
var S: string;
begin
  S:=Socket.ReceiveText;
  if S<>'' then ShowHint(Format('Command <%s> received from %s (%s)',
     [S, Socket.RemoteHost, Socket.RemoteAddress]));
  // execute DM2000 commands here!
  if S='TIMER' then RecordAction.Execute;
  if S='POINT' then TimerTimer(Sender);
  if S='EXPSETTINGS' then SendData('EXPSETTINGS=settings'#13#10+Caption);
end;

///////////// Other events ///////////////////////////////////////////////////

procedure TServerForm.DetachActionExecute(Sender: TObject);
begin
  if ConnectionsSG.Row-1<ServerSocket.Socket.ActiveConnections then
  ServerSocket.Socket.Connections[ConnectionsSG.Row-1].Close;
end;

procedure TServerForm.ListenActionExecute(Sender: TObject);
begin
  if not ServerSocket.Active then
  begin ServerSocket.Port:=StrToInt(PortE.Text); ServerSocket.Open; end
  else ServerSocket.Close;     // if active, click will close ALL connections!
  ListenAction.Update;
end;

procedure TServerForm.FormDestroy(Sender: TObject);
begin ServerSocket.Close; end;

procedure TServerForm.PasswordEChange(Sender: TObject);
begin ListenAction.Enabled:=PasswordE.Text<>''; end;

procedure TServerForm.DetachActionUpdate(Sender: TObject);
var I: integer;
const PrevConnections: integer=0;
begin
  DetachAction.Enabled:=ServerSocket.Socket.ActiveConnections>0;
  if ServerSocket.Socket.ActiveConnections<>PrevConnections then
  begin                       {if number of connections changed, update table}
    PrevConnections:=ServerSocket.Socket.ActiveConnections;
    for I:=0 to ConnectionsSG.RowCount-2 do
    with ServerSocket.Socket do if I<ActiveConnections then
    begin
      ConnectionsSG.Cells[1,I+1]:=Connections[I].RemoteAddress;
      ConnectionsSG.Cells[2,I+1]:=Connections[I].RemoteHost;
    end else
    begin ConnectionsSG.Cells[1,I+1]:=''; ConnectionsSG.Cells[2,I+1]:=''; end;
  end;
end;

procedure TServerForm.ListenActionUpdate(Sender: TObject);
begin
  PasswordE.Enabled:=not ServerSocket.Active;
  PortE.Enabled:=not ServerSocket.Active;
  UpDown.Enabled:=not ServerSocket.Active;
  ListenSB.Down:=ServerSocket.Active;
end;

procedure TServerForm.FormCreate(Sender: TObject);
begin
  ConnectionsSG.Cols[0].Text:='#'#13#10'1'#13#10'2'#13#10'3'#13#10'4';
  ConnectionsSG.Rows[0].Text:='#'#13#10'Address'#13#10'Host';
end;

procedure TServerForm.RecordActionExecute(Sender: TObject);
begin
  Timer.Enabled:=not Timer.Enabled;
  StartSpeedButton.Down:=Timer.Enabled;
end;

procedure TServerForm.RecordActionUpdate(Sender: TObject);
begin
  StartSpeedButton.Down:=Timer.Enabled;
end;

procedure TServerForm.TimerTimer(Sender: TObject);
const Value: integer=0;
begin
  SendData('DATA=2'#13#10'X='+IntToStr(Value)+#13#10'Y='+
  FloatToStr(Random+Value));
  if ScanUpSpeedButton.Down then Inc(Value);
  if ScanDownSpeedButton.Down then Dec(Value);
end;

end.


