'******************************************************************
'*   This script calculates and plots "best fit line" for current *
'* serie. Unlike embedded best fit line available from serie      *
'* properties dialog box it uses only points within selected plot *
'* area. If current serie or plot selection is empty it will      *
'* produce appropriate error message.                             *
'*   This script also demonstrate IDMSerie.GetPoint() member.     *
'******************************************************************

function BestFit

if Server.DocumentCount=0 then bestFit="No document active!" : exit function

set doc=Server.ActiveDocument
set plot=doc.Plot
set ser=doc.Plot.CurrentSerie

if ser.IsEmpty then bestFit="Current serie is empty!" : exit function

dim x,y,v1,v2 'item buffers
n=0 ' counter
bfX=0 ' best fit sums
bfY=0 
bfXY=0
bfX2=0
dim x1,x2,y1,y2 ' plot selection
dim bfA,bfB ' line coefficients

x1=plot.SelectionLeft
x2=plot.SelectionRight
y1=plot.SelectionBottom
y2=plot.SelectionTop

for i=ser.FirstLine to ser.LastLine
  ser.GetPoint i,x,y
  if ((x>=x1) and (x<=x2) and (y>=y1) and (y<=y2)) then
     n=n+1 
     bfX=bfX+x
     bfY=bfY+y
     bfXY=bfXY+x*y
     bfX2=bfX2+x*x
  end if
next

if n=0 then bestFit="No points selected!" : exit function

bfA=n*bfX2-bfX*bfX
if bfA=0 then bestFit="Unable to calculate best fit!" : exit function
bfA=(n*bfXY-bfX*bfY)/bfA
bfB=(bfY-bfA*bfX)/n

' show best fit in last serie. If it's nonempty, add new
set ser=plot.Series(plot.SerieCount-1)
if not ser.IsEmpty then
  plot.AddSerie()
  set ser=plot.Series(plot.SerieCount-1)
end if
ser.IsFunction=true
ser.Text="Best fit"
ser.YExpression="cx*" & bfA & "+" & bfB

Server.DisplayMessage(ser.YExpression)
Server.Notes.AddLine(vbCrLf & "--------------------------------")
Server.Notes.AddLine("x1: " & x1 & "; x2: " & x2)
Server.Notes.AddLine("Fit: Y=X*" & bfA & "+" & bfB)

end function