' Script should be used to process Rhall data from spin-sample inset
'
' Input: file with Step, Rhall, Temp columns
' 
' First: build series. Start new serie when step sign changed
' Note: series divided by points with equal steps, e.g
' 4->2->0-> {start new!} 0->2->4, or 198->200 {new} 200->198->...
'
' Second: fit serie! routine adds fit parameters to given window
' columns: p1..p5; function FitSerie() should NOT be called manually!
'

'####################################################################

sub BuildSeries()
dim Plt,Cont,I,B,Delta,D1,D2,S1,S2,AvTemp

set Plt=Server.ActiveDocument.Plot
set Cont=Server.ActiveDocument.Container

' Clear all series
for I=0 to Plt.SerieCount-1
    Plt.Series(I).ClearSelection
next

Plt.SerieIndex=0   ' start fill from zero serie
I=0                ' other initialization
B=0
AvTemp=0
D1=Cont.Items(0)
D2=Cont.Items(1)
Delta=D2(0)-D1(0)

do                 ' MAIN cycle
  D1=Cont.Items(I)
  D2=Cont.Items(I+1)
  I=I+1
  S1=D1(0)
  S2=D2(0)
  AvTemp=AvTemp+D1(2)  ' accumulate temperature
  if (Delta<>(S2-S1)) and (S2<>S1) or (I=Cont.ItemCount-1) then
     Delta=S2-S1
     Plt.CurrentSerie.XColumn=1
     Plt.CurrentSerie.YColumn=2
     Plt.CurrentSerie.FirstLine=B
     Plt.CurrentSerie.LastLine=I-2
     Plt.CurrentSerie.Text="T=" & Round(AvTemp/(I-B-1),2)
     AvTemp=0
     B=I-1
     if (Plt.SerieIndex=Plt.SerieCount-1) and (I<Cont.ItemCount-1) then
        Plt.AddSerie
     end if
     Plt.SerieIndex=Plt.SerieIndex+1
  end if
loop while I<Cont.ItemCount-1

end sub

'####################################################################

function FitSerie(Ser,ResCont,FitSer)
dim X,Y,Params(4),Sigma(4),Expr,NIter,ChiSqr,ResCode
dim I,FL,Cont,CX,CY,D,S,NumPoints,Output(5)

NIter=40
Expr="p1+p2*sin(2*pi*cx/200+p3)+p4*sin(4*pi*cx/200+p5)"
AvTemp=0

for I=0 to 4
  Params(I)=1
  Sigma(I)=0 
next

set Cont=Ser.Container
FL=Ser.FirstLine
NumPoints=Ser.LastLine-FL+1
CX=Ser.XColumn-1
CY=Ser.YColumn-1
ReDim X(NumPoints-1)         ' Redim(upper index=np-1!)
ReDim Y(NumPoints-1)

Params(0)=0                  ' precalculate first par-r

for I=1 to NumPoints         ' copy data being fitted
  D=Cont.Items(FL+I-1)
  X(I-1)=D(CX)
  Y(I-1)=D(CY)
  Params(0)=Params(0)+D(CY)
  Output(0)=Output(0)+D(2)   ' accumulate temperature
next

Params(0)=Params(0)/NumPoints  ' initial suggestion for P1
Output(0)=Output(0)/NumPoints  ' average temperature

call Server.LMFit(X,Y,0,0,Expr,Params,Sigma,NIter,ChiSqr,ResCode)

S=""                           ' form text (debug?) output
for I=1 to 5
  S=S & "P[" & I & "]=" & Params(I-1) & " Dev=" & Sigma(I-1) & vbCrLf
next

S=S & "Rescode=" & ResCode & vbCrLf & "ChiSqr=" & ChiSqr 
FitSerie=S & vbCrLf & "NIter=" & NIter

for I=0 to 4                   ' add point to output container
  Output(I+1)=Params(I) 
next
ResCont.AddItem(Output)
ResCont.Modified=true

FitSer.IsFunction=true         ' make fit serie

S=Params(0) & "+" & Params(1) & "*sin(2*pi*cx/200+" & Params(2)
S=S & ")+" & Params(3) & "*sin(4*pi*cx/200+" & Params(4) & ")"

FitSer.YExpression=S

end function

'####################################################################

sub FitAll
dim I,SWin,RWin,Plt,S,NumSer
  S=Server.ActiveDocument.WindowCaption
  set Plt=Server.ActiveDocument.Plot
  NumSer=Plt.SerieCount 
  set RWin=Server.CreateDocument("")
  RWin.WindowCaption=S & " fitting results"
  for I=1 to 5 
    RWin.Worksheet.ColumnLabels(I+1)="P" & I
  next
  RWin.Worksheet.ColumnLabels(1)="T, K"
  for I=0 to Plt.SerieCount-1
    if not Plt.Series(I).IsEmpty then
      Plt.AddSerie
      S=FitSerie(Plt.Series(I),RWin.Container,Plt.Series(NumSer))
      Plt.Series(NumSer).Text="Fit of " & Plt.Series(I).Text
      NumSer=NumSer+1
    end if
  next  
end sub
